#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ===============================================================================
#                   SelectiveBrightness.py V1 - (c) Berthold Hinz 2010
# ===============================================================================
#
#    Needs at least Python 2.6.4!
#
# ===============================================================================
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details: <http://www.gnu.org/licenses/>.
#
# ===============================================================================


import pygtk
import gtk
import gtk.glade
import colorsys
import os
import gimp, gimpplugin
from gimpenums import *
import sys
pdb = gimp.pdb

class gui:
    def __init__(self):
        gladefile=sys.path[0] +"/"+"SelectiveBrightness.gui"
        self.wTree=gtk.glade.XML(gladefile)

        dic={"on_cancel": (gtk.main_quit),
                "on_window1_show":window1_show,
                "on_hscaleMT_value_changed":self.value_changed,
                "on_hscaleSH_value_changed":self.value_changed,
                "on_hscaleHL_value_changed":self.value_changed,
                "on_buttonOK_clicked":self.buttonOK_clicked,
                "on_buttonReset_clicked":self.buttonReset_clicked,
                "on_buttonZero_clicked":self.buttonZero_clicked,
                "on_buttonCancel_clicked":self.buttonCancel_clicked,
                "on_buttonFlatten_clicked":self.buttonFlatten_clicked,
                "on_window1_show":window1_show}

        self.window1=self.wTree.get_widget("window1")
        self.label1=self.wTree.get_widget("label1")
        self.label2=self.wTree.get_widget("label2")
        self.label3=self.wTree.get_widget("label3")
        self.hscaleSH=self.wTree.get_widget("hscaleSH")
        self.hscaleMT=self.wTree.get_widget("hscaleMT")
        self.hscaleHL=self.wTree.get_widget("hscaleHL")
        self.buttonOK=self.wTree.get_widget("buttonOK")
        self.buttonCancel=self.wTree.get_widget("buttonCancel")

        self.wTree.signal_autoconnect(dic)
        self.win = self.wTree.get_widget("window1")
        self.win.show()

        if drw<>img.layers[0]:
            self.window1.set_sensitive(False) 
            dialog = gtk.MessageDialog(self.win,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, None)
            dialog.set_markup('The Selective Brightness plugin works on the top layer of the image, but currently the active layer is not the top one.<b>\n\nDo you want to switch to the top layer now?</b>') 
            dialog.connect("destroy", lambda w: self.window1.set_sensitive(True))
            dialog.set_title("Selector: Question") 
            answer = dialog.run()
            dialog.destroy()
            if answer == -8: # YES
                dialog.destroy()
                pdb.gimp_image_set_active_layer(img,img.layers[0])
            else: # NO
                dialog.destroy()
                self.window1.destroy()
                return

        go()
        global blocktop
        global currSH
        global currMT
        global currHL
        currSH=float(img.layers[blocktop+4].opacity-img.layers[blocktop+5].opacity)
        self.hscaleSH.set_value(currSH)
        currMT=float(img.layers[blocktop+2].opacity-img.layers[blocktop+3].opacity)
        self.hscaleMT.set_value(currMT)
        currHL=float(img.layers[blocktop].opacity-img.layers[blocktop+1].opacity)
        self.hscaleHL.set_value(currHL)

    def buttonReset_clicked(self,widget):
        global currSH
        global currMT
        global currHL
        self.hscaleSH.set_value(currSH)
        self.hscaleMT.set_value(currMT)
        self.hscaleHL.set_value(currHL)

    def buttonZero_clicked(self,widget):
        self.hscaleSH.set_value(0)
        self.hscaleMT.set_value(0)
        self.hscaleHL.set_value(0)

    def buttonCancel_clicked(self,widget):
        global blocktop
        global currSH
        global currMT
        global currHL
        global alreadydone
        self.hscaleSH.set_value(currSH)
        self.hscaleMT.set_value(currMT)
        self.hscaleHL.set_value(currHL)
        if alreadydone==False:
            for i in xrange(6):
                pdb.gimp_image_remove_layer(img,img.layers[0])
                pdb.gimp_displays_flush()
        pdb.gimp_image_undo_group_end(img)
        self.window1.destroy()
        return

    def buttonFlatten_clicked(self,widget):
        SH=int(img.layers[blocktop+4].opacity-img.layers[blocktop+5].opacity)
        MT=int(img.layers[blocktop+2].opacity-img.layers[blocktop+3].opacity)
        HL=int(img.layers[blocktop+0].opacity-img.layers[blocktop+1].opacity)
        layer_SB=pdb.gimp_layer_new_from_visible(img,img,"Selective Brightness S"+str(SH) + " M" +str(MT) + " H" + str(HL) )

        img.add_layer(layer_SB, -1)
        for i in xrange(6):
            pdb.gimp_image_remove_layer(img,img.layers[1])
            pdb.gimp_displays_flush()
        pdb.gimp_image_undo_group_end(img)
        self.window1.destroy()
        return

    def buttonOK_clicked(self,widget):
        SH=int(img.layers[blocktop+4].opacity-img.layers[blocktop+5].opacity)
        MT=int(img.layers[blocktop+2].opacity-img.layers[blocktop+3].opacity)
        HL=int(img.layers[blocktop+0].opacity-img.layers[blocktop+1].opacity)
        layer_SB=pdb.gimp_layer_new_from_visible(img,img,"Selective Brightness S"+str(SH) + " M" +str(MT) + " H" + str(HL) )
        img.add_layer(layer_SB, -1)
        pdb.gimp_displays_flush()
        pdb.gimp_image_undo_group_end(img)
        self.window1.destroy()
        return

    def value_changed(self,widget):
        global blocktop
        if widget.name=="hscaleSH": 
            lnr=4+blocktop
        if widget.name=="hscaleMT": 
            lnr=2+blocktop
        if widget.name=="hscaleHL": 
            lnr=0+blocktop
        val=widget.get_value()
        if val>=0:
            val_l=val
            val_d=0
        else:
            val_l=0
            val_d=-1*val
        img.layers[lnr+1].opacity=float(val_d)
        img.layers[lnr].opacity=float(val_l)
        pdb.gimp_displays_flush()

def go():

    alreadydone=False
    global blocktop
    global alreadydone
    blocktop=0
    lcount=len(img.layers)
    pdb.gimp_image_undo_group_start(img)
    if lcount>=7:
        for i in xrange(lcount):
            if img.layers[i].name=="Światło jaśniesze" and img.layers[i+1].name=="Highlights darker" and i+6<= lcount:
                blocktop=i
                alreadydone=True

    if alreadydone==True:
        if img.layers[blocktop+1].name<>"Światło ciemniejsze": alreadydone=False
        if img.layers[blocktop+2].name<>"Półcienie jaśniesze": alreadydone=False
        if img.layers[blocktop+3].name<>"Półcienie ciemniejsze": alreadydone=False
        if img.layers[blocktop+4].name<>"Cienie jaśniesze": alreadydone=False
        if img.layers[blocktop+5].name<>"Cienie ciemniejsze": alreadydone=False

    if alreadydone==True:
        for i in xrange(blocktop):
            print i
            img.layers[i].visible=False
    else:
        for i in xrange(blocktop):
            print i
            img.layers[i].visible=False
        bgc = gimp.get_background()
        fgc = gimp.get_foreground()
        gimp.set_background(0,0,0)
        gimp.set_foreground(255,255,255)
        for i in xrange(6):
            layer_copy = drw.copy(True)
            layer_copy.mode = OVERLAY_MODE
            layer_copy.name = "SB"
            layer_copy.opacity = 0.0
            img.add_layer(layer_copy, -1)

        for i in xrange(2):
            l=i+4
            points = (0, 255, 32, 255 , 96,0, 255,0)
            pdb.gimp_curves_spline(img.layers[l], 0, 8, points)
            mask = pdb.gimp_layer_create_mask(img.layers[l], 5)
            pdb.gimp_layer_add_mask(img.layers[l],mask)
            if i==0:
                img.layers[i+4].name="Cienie jaśniesze"
                pdb.gimp_edit_fill(img.layers[l],0)
            else:
                img.layers[i+4].name="Cienie ciemniejsze"
                pdb.gimp_edit_fill(img.layers[l],1)

        for i in xrange(2):
            l=i+2
            points = (0,0,80,0, 128, 255 ,175, 0,255,0)
            pdb.gimp_curves_spline(img.layers[l], 0, 10, points)
            pdb.gimp_invert(img.layers[l])
            mask = pdb.gimp_layer_create_mask(img.layers[l], 5)
            pdb.gimp_layer_add_mask(img.layers[l],mask)
            if i==0:
                img.layers[l].name="Półcienie jaśniesze"
                pdb.gimp_edit_fill(img.layers[l],0)
            else:
                img.layers[l].name="Półcienie ciemniejsze"
                pdb.gimp_edit_fill(img.layers[l],1)

        for i in xrange(2):
            l=i
            points = (0, 0, 160,0, 224,255 , 255,255)
            pdb.gimp_curves_spline(img.layers[l], 0, 8, points)
            mask = pdb.gimp_layer_create_mask(img.layers[l], 5)
            pdb.gimp_layer_add_mask(img.layers[l],mask)
            if i==0:
                img.layers[l].name="Światło jaśniesze"
                pdb.gimp_edit_fill(img.layers[l],0)
            else:
                img.layers[l].name="Światło ciemniejsze"
                pdb.gimp_edit_fill(img.layers[l],1)

        gimp.set_background(bgc)
        gimp.set_foreground(fgc)

def window1_show(widget):
    pass

def hscaleMT_value_changed(widget):
    pass

def hscaleHL_value_changed(widget):
    pass

def buttonOK_clicked(widget):
    pass

def buttonCancel_clicked(widget):
    pass

class SelectiveBrightness_plugin(gimpplugin.plugin):
    def start(self):
        gimp.main(self.init, self.quit, self.query, self._run)

    def init(self):
        pass

    def query(self):
        author = "Berthold Hinz"
        copyright = "Berthold Hinz"
        menu = "<Image>/Colors/Selektywna regulacja jasności"
        year = "2010"
        description = "Descriptiontext"
        help = "Helptext"
        parameters = [
            (PDB_INT32, "run_mode","Run mode"),
            (PDB_IMAGE, "image","Input image"),
            (PDB_DRAWABLE, "drawable","Input drawable"),
            ]
        gimp.install_procedure("SelectiveBrightnessGUI",
            description,
            help,
            author,
            copyright,
            year,
            menu,
            "*",
            PLUGIN,
            parameters,
            [])

    def SelectiveBrightnessGUI(self, run_mode, Image, drawable):
        global img
        img = Image
        global drw
        drw = drawable
        App = gui()
        gtk.main()

if __name__ == '__main__':
    SelectiveBrightness_plugin().start()
